searchNodes=[{"ref":"Validate.html","title":"Validate","module":"Validate","type":"module","doc":"Validate, validate incoming requests in an easy to reason-about way."},{"ref":"Validate.html#validate/2","title":"Validate.validate/2","module":"Validate","type":"function","doc":"Validates an input against a given list of rules Examples iex&gt; {:ok, _data} = Validate.validate(&quot;Jane&quot;, required: true, type: :string) iex&gt; {:error, _errors} = Validate.validate([], required: true, type: :list) iex&gt; {:ok, _data} = Validate.validate(%{&quot;name&quot; =&gt; &quot;Jane&quot;}, %{&quot;name&quot; =&gt; [required: true, type: :string]})"},{"ref":"Validate.Plugs.FormRequest.html","title":"Validate.Plugs.FormRequest","module":"Validate.Plugs.FormRequest","type":"module","doc":""},{"ref":"Validate.Rules.DateString.html","title":"Validate.Rules.DateString","module":"Validate.Rules.DateString","type":"module","doc":""},{"ref":"Validate.Rules.DateString.html#validate/1","title":"Validate.Rules.DateString.validate/1","module":"Validate.Rules.DateString","type":"function","doc":""},{"ref":"Validate.Util.html","title":"Validate.Util","module":"Validate.Util","type":"module","doc":"Utilities to help with validation"},{"ref":"Validate.Util.html#errors_to_map/1","title":"Validate.Util.errors_to_map/1","module":"Validate.Util","type":"function","doc":"Turns an array of %Validate.Validator.Error{} structs into a keyed map of errors Examples iex&gt; {:error, errors} = Validate.validate(%{&quot;name&quot; =&gt; &quot;&quot;}, %{&quot;name&quot; =&gt; [required: true]}) iex&gt; %{&quot;name&quot; =&gt; [&quot;is required&quot;]} == Validate.Util.errors_to_map(errors)"},{"ref":"Validate.Validator.html","title":"Validate.Validator","module":"Validate.Validator","type":"module","doc":"Provides the Error and Arg structs, as well as helpers for validation responses"},{"ref":"Validate.Validator.html#error/1","title":"Validate.Validator.error/1","module":"Validate.Validator","type":"function","doc":"Used in validators to return an error with a message"},{"ref":"Validate.Validator.html#halt/0","title":"Validate.Validator.halt/0","module":"Validate.Validator","type":"function","doc":"Used in validators to prevent further validation for a field but treated as a success"},{"ref":"Validate.Validator.html#halt/1","title":"Validate.Validator.halt/1","module":"Validate.Validator","type":"function","doc":"Used in validators to prevent further validation for a field while also adding an error message"},{"ref":"Validate.Validator.html#success/1","title":"Validate.Validator.success/1","module":"Validate.Validator","type":"function","doc":"Used in validators to indicate validation was successful for a field"},{"ref":"readme.html","title":"Validate","module":"readme","type":"extras","doc":"Validate Validate data in elixir. Focused on validating incoming http request data but works with all elixir data types. Coming from languages like PHP and Node.js it can be difficult to reason about validating your requests using Ecto. This provides a simple data validation layer that aims to be extensible to allow for custom validation logic provided by the user. Inspired by Laravel&#39;s validator."},{"ref":"readme.html#installation","title":"Validate - Installation","module":"readme","type":"extras","doc":"Add Validate to your mix.exs dependencies: defp deps do [ {:validate, &quot;~&gt; 1.3&quot;} ] end"},{"ref":"readme.html#features","title":"Validate - Features","module":"readme","type":"extras","doc":"[x] Simple (raw string, bool, etc), nested maps, and list validation [x] Built in validation rules [x] Custom validation rules [x] Return validated keys only [x] Plug support [ ] i18n"},{"ref":"readme.html#table-of-contents","title":"Validate - Table of Contents","module":"readme","type":"extras","doc":"Installation Usage Simple Validation Map Validation Nested Rules Errors Formatting Errors Custom Validation Rules Plug Built in Validation Rules"},{"ref":"readme.html#usage","title":"Validate - Usage","module":"readme","type":"extras","doc":"The basic usage looks like: # data is the validated input {:ok, data} = Validate.validate(input, rules) # errors is an array of %Validate.Validator.Error{} {:error, errors} = Validate.validate(input, rules) Simple Validation If you are validating a simple/flat data type (like string, boolean, number), you can simply pass the rules as a keyword list: {:ok, data} = Validate.validate(&quot;jane&quot;, required: true, type: :string) Map Validation If you are validating a map, such as the body of an incoming post request, you must pass a map of rules where the keys match the keys that you are validating. Note: the rule keys must be the same type as the data keys. If your data keys are strings, use string keys for the rules. If your data keys are atoms, use atoms for the rules. You can mix and match key types in the same map, as long as the keys in the rules mirror the keys in the data. input = %{ &quot;email&quot; =&gt; &quot;test@example.com&quot;, &quot;age&quot; =&gt; 32 } rules = %{ &quot;email&quot; =&gt; [required: true, type: :string], &quot;age&quot; =&gt; [required: true, type: :number] } {:ok, data} = Validate.validate(input, rules) Important: The data returned from nested validations will always only be the keys specified in the rules. Any keys not specified in the rules are thrown away: input = %{ &quot;url&quot; =&gt; &quot;https://google.com&quot;, &quot;friends&quot; =&gt; [ %{ &quot;name&quot; =&gt; &quot;Jane&quot;, &quot;email&quot; =&gt; &quot;jane@example.com&quot; }, ], &quot;age&quot; =&gt; 28 } rules = %{ &quot;url&quot; =&gt; [required: true, type: :string], &quot;friends&quot; =&gt; [ map: %{ &quot;name&quot; =&gt; [required: true, type: :string] } ] } {:ok, data} = Validate.validate(input, rules) data == %{ &quot;url&quot; =&gt; &quot;https://google.com&quot;, &quot;friends&quot; =&gt; [ %{ &quot;name&quot; =&gt; &quot;Jane&quot;} ] } Nested Rules You can also perform complex validation such as nested lists and maps: input = %{ &quot;name&quot; =&gt; &quot;Jane Doe&quot;, &quot;colors&quot; =&gt; [&quot;red&quot;, &quot;green&quot;], &quot;address&quot; =&gt; %{ &quot;line1&quot; =&gt; &quot;123 Fake St&quot;, &quot;city&quot; =&gt; &quot;Saskatoon&quot; }, &quot;friends&quot; =&gt; [ %{&quot;name&quot; =&gt; &quot;John&quot;, &quot;email&quot; =&gt; &quot;john@example.com&quot;}, %{&quot;name&quot; =&gt; &quot;Michelle&quot;, &quot;email&quot; =&gt; &quot;michelle@example.com&quot;} ] } rules = %{ &quot;name&quot; =&gt; [required: true, type: :string], &quot;colors&quot; =&gt; [ required: true, type: :list, list: [required: true, type: :string, in: ~w[blue orange red green purple]] ], &quot;address&quot; =&gt; [ required: true, type: :map, map: %{ &quot;line1&quot; =&gt; [required: true, type: :string], &quot;city&quot; =&gt; [required: true, type: :string], } ], &quot;friends&quot; =&gt; [ required: true, type: :list, list: [ required: true, type: :map, map: %{ &quot;name&quot; =&gt; [required: true, type: :string], &quot;email&quot; =&gt; [required: true, type: :string] } ] ] } {:ok, data} = Validate.validate(input, rules)"},{"ref":"readme.html#errors","title":"Validate - Errors","module":"readme","type":"extras","doc":"The error array returned contains %Validate.Validator.Error{} structs. When performing simple validations, path will be an empty array. When validating maps, path will be an array of keys representing the depth to get to the data value. For lists, the numeric index is used for the specific position of the error in the list. [ %Validate.Validator.Error{ path: [&quot;name&quot;], message: &quot;is required&quot;, rule: :required }, %Validate.Validator.Error{ path: [&quot;colors&quot;, 2], message: &quot;must be a string&quot;, rule: :type }, %Validate.Validator.Error{ path: [&quot;address&quot;, &quot;line1&quot;], message: &quot;is required&quot;, rule: :required }, ] Formatting Errors Sometimes it is easier to show errors to the user by name instead of looping through the array, so a helper is provided to make this easy: # assuming the error array from the above example formatted = Validate.Util.errors_to_map(errors) formatted == %{ &quot;name&quot; =&gt; [&quot;is required&quot;], &quot;colors.2&quot; =&gt; [&quot;must be a string&quot;], &quot;address.line1&quot; =&gt; [&quot;is required&quot;] }"},{"ref":"readme.html#custom-validation-rules","title":"Validate - Custom Validation Rules","module":"readme","type":"extras","doc":"You can write your own validators by using the custom: rule. You can write these as modules or as inline functions. Validation rules must return one of four possible outcomes: Validate.Validator.success(value) - Return this when validation passes. You must pass the value back as the first arg. This gives you the opportunity to transform the value if you wish. Validate.Validator.error(message) - Return this when validation fails. You must pass a message back to the user to let them know why it failed. Validate.Validator.halt(message) - Return this when validation fails AND you want to prevent any further validation from running on this specific input. Validate.Validator.halt() - Return this when validation succeeds BUT you do not want any further validations to happen on this specific input. For example, we could write a custom rule to see if an email already exists in our database: input = %{ &quot;email&quot; =&gt; &quot;hello@example.com&quot; } rules = %{ &quot;email&quot; =&gt; [ required: true, type: :string, custom: fn %{ value: value } -&gt; if MyApp.Accounts.email_exists?(value) do Validate.Validator.error(&quot;email is already taken&quot;) else Validate.Validator.success(value) end end ] } If you&#39;d like to have the rule be reusable, you could move it into a module. For further readability, you could also make it a higher-order function to prevent passing function references like &amp;MyApp.MyRule.validate/1: defmodule MyApp.EmailExists do def validate() do fn %{ value: value } -&gt; if MyApp.Accounts.email_exists?(value) do Validate.Validator.error(&quot;email is already taken&quot;) else Validate.Validator.success(value) end end end end input = %{ &quot;email&quot; =&gt; &quot;hello@example.com&quot; } rules = %{ &quot;email&quot; =&gt; [ required: true, type: :string, custom: MyApp.EmailExists.validate(), ] }"},{"ref":"readme.html#plug","title":"Validate - Plug","module":"readme","type":"extras","doc":"You can use Validate as a plug to simplify your controllers by performing validation and authorization outside of your controller action. First you need to create a file in your project for the plug. This is required so that you can control what happens when different events occur. Some applications may wish to return JSON, while others want to redirect to a route. Providing this as a macro in your own application gives you the flexibility to handle this how you please. For this example, we&#39;ll create a file called my_app_web/plugs/form_request.ex defmodule MyAppWeb.Plugs.FormRequest do use Validate.Plugs.FormRequest, validate_success: :validate_success, validate_error: :validate_error, auth_error: :auth_error import Plug.Conn # called when validation passess # use this to make the validated data available to your controller def validate_success(conn, data) do conn |&gt; assign(:validated, data) end def validate_error(conn, errors) do error_map = Validate.Util.errors_to_map(errors) conn |&gt; put_status(422) |&gt; Phoenix.Controller.json(%{errors: error_map}) |&gt; halt() end def auth_error(conn) do conn |&gt; put_status(401) |&gt; Phoenix.Controller.json(%{message: &quot;unauthorized&quot;}) |&gt; halt() end end Now we can create a module that will handle the request. This module can exist anywhere but needs to implement at least a rules/1 function and optionally an authorize/1 function. authorize/1 will be called first if it is present, and only if it returns a truthy value will it continue to validate the input, if it returns a falsy value we will call the auth_error function in our plug from earlier. rules/1 should return a map of rules that will be validated against conn.params. Let&#39;s pretend we&#39;re making a form to create a team in our application. We might put this in my_app_web/requests/store_team_request.ex. defmodule MyAppWeb.Requests.StoreTeamRequest do # can only create a team if we do not already have a team def authorize(conn) do conn.assigns.user &amp;&amp; !conn.assigns.team end def rules(_conn) do %{ &quot;name&quot; =&gt; [required: true, type: :string, max: 80], &quot;plan&quot; =&gt; [required: true, type: :string, in: [&quot;free&quot;, &quot;pro&quot;]], # ... } end end Now we&#39;re ready to use this in our controller. Let&#39;s assume we have a controller like my_app_web/controllers/team_controller.ex. defmodule MyAppWeb.Controllers.TeamController do use MyAppWeb, :controller plug MyAppWeb.Plugs.FormRequest, MyAppWeb.Requests.StoreTeamRequest when action in [:store] def store(conn, params) do # now we can operate in here knowing that # our request is authorized and the validated data # is in conn.assigns.validated end end The controller can be registered like normal in your routes file, and the plug will take care of authorizing and validating before calling the controller action! # inside your routes file somewhere scope &quot;/&quot; do pipe_through [:browser] post &quot;/teams&quot;, MyAppWeb.Controllers.TeamController, :store end If you wanted, you could bypass the controller all together by adding a perform/2 function to the request module and just handle the request right there: defmodule MyAppWeb.Requests.StoreTeamRequest do def rules(_conn) do %{ &quot;name&quot; =&gt; [required: true, type: :string, max: 80], # ... } end def perform(conn, data) do # skip the controller and just handle the request here # `data` is the validated data end end # then in your router file scope &quot;/&quot; do pipe_through [:browser] post &quot;/teams&quot;, MyAppWeb.Plugs.FormRequest, MyAppWeb.Requests.StoreTeamRequest end If you choose to use perform/2 then the validate_success option provided in plugs/form_request.ex will be ignored for that request."},{"ref":"readme.html#built-in-validation-rules","title":"Validate - Built in Validation Rules","module":"readme","type":"extras","doc":"Examples show input that would successfully validate. Some rules require a type to be specified first in order to properly validate. comparison operators (==, &gt;, &gt;=, &lt;, &lt;=) between cast characters date_string digits_between digits ends_with in ip/v4/v6 lowercase max_digits max min_digits min not_ends_with not_in not_regex not_starts_with nullable regex required size starts_with type uppercase url uuid comparison operators You can use atoms of ==, &gt;, &gt;=, &lt;, and &lt;= to compare values. Aside from primitive values you can compare Date, DateTime, and NaiveDateTime values. Validate.validate(&quot;bob&quot;, ==: &quot;bob&quot;) Validate.validate(~D[2020-02-15], &gt;: ~D[2015-10-14]) between The field under validation must be between the two numbers (inclusive). Validate.validate(10, type: :number, between: {1, 20}) cast Trys to convert the field to the specified type. Can be used in place of the type rule if you e.g. want a number as the end result but are taking input from an HTML form which is inputted as a string. Later validation rules and the resulting validated data will contain the casted value in the new type. Prevents further rules from running if validation fails. Available types are: :atom, :boolean, :float, :integer, :string. Validate.validate(&quot;123&quot;, cast: :integer) nil is converted to the appropriate &quot;empty&quot; value of the type: :atom -&gt; :&quot;&quot; :boolean -&gt; false :float -&gt; 0.0 :integer -&gt; 0 :string -&gt; &quot;&quot; If you&#39;d like to preserve nil you must use the full syntax and pass the :preserve flag: Validate.validate(nil, cast: [to: :integer, nil: :preserve]) characters The field under validation must have a specific set of characters. Useful for things like usernames. # letters only Validate.validate(&quot;LeslieKnope&quot;, type: :string, characters: :alpha) # letters + numbers Validate.validate(&quot;LeslieKnope123&quot;, type: :string, characters: :alpha_num) # letters, numbers, dashes, underscores Validate.validate(&quot;Leslie-Knope_123&quot;, type: :string, characters: :alpha_dash) Each option also has an _ascii alternative available to restrict any non-ascii characters: :alpha_ascii, :alpha_num_ascii, :alpha_dash_ascii. date_string The field under validation must be a string matching a valid Timex string format. Validate.validate(&quot;2020-02-10&quot;, date_string: &quot;{YYYY}-{0M}-{0D}&quot;) digits_between The field under validation must have a number of digits between the two numbers (inclusive). Validate.validate(100, type: :integer, digits_between: {2, 4}) digits The field under validation must have the exact amount of digits. Validate.validate(100, type: :integer, digits: 3) ends_with The field under validation must end with the specified string. Validate.validate(&quot;exciting!&quot;, type: :string, ends_with: &quot;!&quot;) in The field under validation must be in the provided array. Validate.validate(&quot;blue&quot;, type: :string, in: [&quot;red&quot;, &quot;green&quot;, &quot;blue&quot;]) ip The field under validation must be a valid IP address. # allows v4 or v6 Validate.validate(&quot;127.0.0.1&quot;, type: :string, ip: true) # v4 only Validate.validate(&quot;127.0.0.1&quot;, type: :string, ip: :v4) # v6 only Validate.validate(&quot;3ffe:b80:1f8d:2:204:acff:fe17:bf3&quot;, type: :string, ip: :v6) lowercase The field under validation must be all lowercase. Validate.validate(&quot;hello&quot;, type: :string, lowercase: true) The field under validation must not be all lowercase. Validate.validate(&quot;Hello&quot;, type: :string, lowercase: false) list The field under validation must be a list, and you must specify rules that should be ran for each item in the list. Validate.validate([1, 2, 3], list: [required: true, type: :number]) map The field under validation must be a map, and you must specify validation rules for the keys of the map. Validate.validate(%{&quot;name&quot; =&gt; &quot;Jane&quot;}, map: %{&quot;name&quot; =&gt; [required: true, type: :string]}) # you can also just pass a map directly as the second argument when the input is a map Validate.validate(%{&quot;name&quot; =&gt; &quot;Jane&quot;}, %{&quot;name&quot; =&gt; [required: true, type: :string]}) # nesting also works Validate.validate(%{ &quot;address&quot; =&gt; %{ &quot;line1&quot; =&gt; &quot;123 Fake st&quot; } }, %{ &quot;address&quot; =&gt; [ map: %{ &quot;line1&quot; =&gt; [ required: true, type: :string ] } ] }) max_digits The field under validation must be at most the provided number of digits. Validate.validate(10, type: :integer, max_digits: 2) max The field under validation must be less than or equal to the provided number. Validate.validate(10, type: :number, max: 15) min_digits The field under validation must be at least the provided number of digits. Validate.validate(10, type: :integer, min_digits: 2) min The field under validation must be greater than or equal to the provided number. Validate.validate(10, type: :number, min: 1) not_ends_with The field under validation must not end with the provided string. Validate.validate(&quot;exciting&quot;, type: :string, not_ends_with: &quot;!&quot;) not_in The field under validation must not be in the provided array. Validate.validate(&quot;black&quot;, type: :string, not_in: [&quot;red&quot;, &quot;green&quot;, &quot;blue&quot;]) not_regex The field under validation must not match the provided pattern. Validate.validate(&quot;abcd&quot;, type: :string, not_regex: ~r/[0-9]/) not_starts_with The field under validation must not start with the provided string. Validate.validate(&quot;company_123&quot;, type: :string, not_starts_with: &quot;user_&quot;) nullable The field under validation can be nil. If the value is nil, it does not process any more rules for the field. # type &amp; min do not run here Validate.validate(nil, nullable: true, type: :string, min: 10) regex The field under validation must match the provided pattern. Validate.validate(&quot;1234&quot;, type: :string, regex: ~r/[0-9]/) required The field under validation must not be empty. Empty values are nil, &quot;&quot;, %{}, [], {}. Validate.validate(&quot;red&quot;, required: true) If set to required: false, empty values will be accepted. Validate.validate(&quot;&quot;, required: false) This can be used as a &quot;required if&quot; by evaluating a statement to a boolean: Validate.validate(&quot;red&quot;, required: check_if_color_is_needed()) size The field under validation must have a size matching the given value. Numbers are compared directly to the value String sizes are calculated with String.length/1 List and map sizes are calculated with Enum.count/1 Tuple sizes are calculated with tuple_size/1 Validate.validate(&quot;123&quot;, size: 3) Validate.validate(10, size: 10) Validate.validate([1, 2, 3], size: 3) Validate.validate(%{ ok: true }, size: 1) Validate.validate({1, 2}, size: 2) starts_with The field under validation must start with the given value. Validate.validate(&quot;user_1234&quot;, type: :string, starts_with: &quot;user_&quot;) type The field under validation must be of the specified type. Valid types are: atom, binary, boolean, date, datetime, float, function, integer, list, map, naive_datetime, number, string, tuple. Validate.validate(:user, type: :atom) uppercase The field under validation must be all uppercase. Validate.validate(&quot;HELLO&quot;, type: :string, uppercase: true) The field under validation must not be all uppercase. Validate.validate(&quot;hEllO&quot;, type: :string, uppercase: false) url The field under validation must be a valid url. Evaluated using URI.new/1 and requires that the url has scheme (such as http://). Validate.validate(&quot;https://example.com&quot;, type: :string, url: true) uuid The field under validation must be a uuid. Validate.validate(&quot;123e4567-e89b-12d3-a456-426655440000&quot;, type: :string, uuid: true)"}]